<?php
/**
 * @file
 * Administrative page callbacks for the Comment Country module.
 */

/**
 * Comment_country administration settings.
 *
 * @return
 *   Form for comment_country configuration options.
 */
function comment_country_settings($form, &$form_state) {
  $form['comment_country_ipinfodb_key'] = array(
    '#type' => 'fieldset',
    '#title' => t('IPinfoDB.com API Key'),
  );
  $form['comment_country_ipinfodb_key']['comment_country_ipinfodb_apikey'] = array(
    '#type' => 'textfield',
    '#size' => 100,
    '#description' => t('Please provide a valid IPInfoDB.com API key. If you don\'t have one, you can register at <a href="@ipinfo">IPInfoDB.com</a>.', array('@ipinfo' => url('http://www.ipinfodb.com/register.php'))),
    '#default_value' => variable_get('comment_country_ipinfodb_apikey', ''),
  );
  $form['comment_country_append'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatically append country information to comments'),
    '#description' => t('<strong>Note:</strong> Appending to \'submitted by\' requires that the active theme uses the standard \'$submitted\' variable in comment.tpl.php.'),
  );
  $form['comment_country_append']['comment_country_append_location'] = array(
    '#type' => 'select',
    '#title' => 'Location of Appended Country Information',
    '#default_value' => variable_get('comment_country_append_location', 'comment body'),
    '#options' => array(
      'comment body' => t('Comment Body'),
      'submitted by' => t('Submitted By'),
      'both' => t('Both'),
      'none' => t('None'),
    ),
    '#description' => t('Select where to automatically append country information (themable via the theme_comment_country_info theme function). Select \'None\' if you plan to print the desired variables in comment.tpl.php directly.'),
  );
  $options = array(
    'country name' => t('Country Name'),
    'country code' => t('Country Code'),
  );
  if (module_exists('countryicons')) {
    $options['country flag'] = t('Country Flag');
  }
  $form['comment_country_append']['comment_country_append_info'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Country Information to Append',
    '#default_value' => variable_get('comment_country_append_info', array('country name')),
    '#options' => $options,
    '#description' => t('Select which information to append (the \'Country Flag\' option will only be available when the <a href="http://drupal.org/project/countryicons">Country Icons</a> module is installed).'),
  );
  $form['comment_country_update'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update country information'),
    '#description' => '<span style="color: red; font-weight: bold;">' . t('WARNING: this will delete all current comment country data. Use with care.') . "</span>",
  );
  $form['comment_country_update']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update Country Information'),
    '#submit' => array('comment_country_admin_submit'),
  );

  return system_settings_form($form);
}

/**
 * Admin settings form submit batch setup handler.
 */
function comment_country_admin_submit($form, &$form_state) {

  if ($form_state['clicked_button']['#value'] == 'Update Country Information') {
    $comment_count = db_query('SELECT COUNT(DISTINCT cid) FROM {comment} WHERE status = 1')->fetchField();

    $batch = array(
      'title' => t('Updating comment country information.'),
      'operations' => array(
        array('comment_country_batch', array($comment_count)),
      ),
      'finished' => 'comment_country_batch_finished',
      'progress_message' => t('Updating country information for comments...'),
      'error_message' => t('Comment Country batch update has encountered an error.'),
      'file' => drupal_get_path('module', 'comment_country') . '/includes/comment_country.admin.inc',
    );

    batch_set($batch);
  }
}

/**
 * Batch callback; update country info for comments.
 */
function comment_country_batch($comment_count, &$context) {
  if (empty($context['sandbox'])) {
    db_query('TRUNCATE TABLE {comment_country}');
    $context['sandbox'] = array();
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_comment'] = 0;
    $context['sandbox']['max'] = $comment_count;
  }

  $query = db_select('comment', 'c')
    ->fields('c', array('cid', 'hostname', 'status'))
    ->orderBy('c.cid', 'ASC')
    ->where('c.cid > :cid', array(':cid' => $context['sandbox']['current_comment']))
    ->condition('c.status', 1);
  $query->range(0, 10);

  $result = $query->execute();
  $ipinfodb_api_key = variable_get('comment_country_ipinfodb_apikey', '');
  foreach ($result as $record) {
    $data = drupal_json_decode(drupal_http_request('http://api.ipinfodb.com/v3/ip-country/?format=json&key=' . $ipinfodb_api_key . '&ip=' . $record->hostname)->data);
    $id = db_insert('comment_country')
      ->fields(array(
        'cid' => $record->cid,
        'country_code' => $data['countryCode'],
        'country_name' => $data['countryName'],
      ))
      ->execute();

    $context['sandbox']['progress']++;
    $context['sandbox']['current_comment'] = $record->cid;
  }

  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = ($context['sandbox']['progress'] / $context['sandbox']['max']);
  }
}


/**
 * Batch callback; count the current number of URL aliases for comparison later.
 */
function comment_country_batch_finished($success, $results, $operations) {
  if ($success) {
    $message = 'Successfully updated country information for all existing published comments.';
  }
  else {
    $error_operation = reset($operations);
    $message = 'An error occurred while processing ' . $error_operation[0] . ' with arguments :' . print_r($error_operation[0], TRUE);
  }
  drupal_set_message(check_plain($message));
}
